package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

/**
 * This interfaces provides a way to digest incoming updates that are commands, e.g. {@code
 * /this_command arg1}
 *
 * @author Davide Polonio
 * @since 1.0
 */
public interface Executor {

  /**
   * Provides the keyword to trigger this executor. Note that it must start with "/" at the
   * beginning, e.g. {@code /start}.
   *
   * @return a {@link String} providing the keyword to trigger the current {@link Executor}
   */
  String getTriggerKeyword();

  /**
   * Process the current update
   *
   * @param chat the chat the {@link Executor} is currently replying to
   * @param update the update to process
   * @return a {@link CompletableFuture} with the result of the computation
   */
  // FIXME cannot be void - we don't want pesky side effects!
  CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update);
}
